#!/bin/sh

check_probe() {
  name=$1
  command=$2
  config_name=$3
  expected_output=$4

  printf '%-25s' $name
  `/usr/libexec/grid-monitoring/probes/pl.plgrid/UNICORE/umi2/$name/$name.pl -f localConfiguration/$name/$config_name.config $command > /tmp/unicore-test/logs/output 2> /dev/null`
  `head -1 /tmp/unicore-test/logs/output | grep -cq "$expected_output"`
  if [ $? -eq 0 ]; then
    echo -n "PASS ("
    echo -n `head -1 /tmp/unicore-test/logs/output`
    echo ")"
  else
    echo -n "FAIL ("
    echo -n `head -1 /tmp/unicore-test/logs/output`
    echo ")"
    cat /tmp/unicore-test/logs/output
    stop_unicore
    exit 1
  fi
}

test_check_gateway() {
  check_probe 'check_gateway' '' 'ok' 'OK: Gateway works properly'
  check_probe 'check_gateway' '' 'warning-missing_vsites' 'WARNING: Required services not found: UNKNOWN-VSITE'
  ./deployment/unicore-servers-*/gateway/bin/stop.sh > /dev/null 2>&1
  check_probe 'check_gateway' '' 'ok' 'CRITICAL: Connection refused'
  ./deployment/unicore-servers-*/gateway/bin/start.sh > /dev/null 2>&1
  sleep 10
}

test_check_uvos() {
  check_probe 'check_uvos' '' 'ok' 'OK: UVOS works, identity matches'
  ./deployment/uvos-*/bin/stopServer.sh > /dev/null 2>&1
  check_probe 'check_uvos' '' 'ok' 'CRITICAL: Unable to contact UVOS'
  sed -i -e's|cn=demo user,o=unicore,c=eu|wrong dn|' deployment/uvos-*/data/hsqlUVOSdb.bin.script
  ./deployment/uvos-*/bin/startServer.sh > /dev/null 2>&1
  sleep 10
  check_probe 'check_uvos' '' 'ok' 'CRITICAL: Access Denied'
  ./deployment/uvos-*/bin/stopServer.sh > /dev/null 2>&1
  sed -i -e's|wrong dn|cn=demo user,o=unicore,c=eu|' deployment/uvos-*/data/hsqlUVOSdb.bin.script
  ./deployment/uvos-*/bin/startServer.sh > /dev/null 2>&1
  sleep 10
}

test_check_registry() {
  check_probe 'check_registry' '' 'missing_site' 'CRITICAL: Some required services not found'
  ./deployment/unicore-servers-*/registry/bin/stop.sh > /dev/null 2>&1
  check_probe 'check_registry' '' 'ok' 'CRITICAL: Unable to connect the Registry'
  mv ./deployment/unicore-servers-*/registry/data /tmp/unicore-test/data
  ./deployment/unicore-servers-*/registry/bin/start.sh > /dev/null 2>&1
  sleep 10
  check_probe 'check_registry' '' 'ok' 'CRITICAL: None services found'
  ./deployment/unicore-servers-*/registry/bin/stop.sh > /dev/null 2>&1
  rm -rf ./deployment/unicore-servers-*/registry/data
  mv /tmp/unicore-test/data ./deployment/unicore-servers-*/registry
  ./deployment/unicore-servers-*/registry/bin/start.sh > /dev/null 2>&1
  sleep 30
  check_probe 'check_registry' '' 'ok' 'OK: Registry works'
}

test_check_unicorex() {
  ./deployment/unicore-servers-*/unicorex/bin/stop.sh > /dev/null 2>&1
  check_probe 'check_unicorex' '' 'ok' 'CRITICAL: Unable to find requested TSF on accessible sites list'
  rm -rf ./deployment/unicore-servers-*/unicorex/data/*
  ./deployment/unicore-servers-*/unicorex/bin/start.sh > /dev/null 2>&1
  sleep 20
  check_probe 'check_unicorex' '' 'ok' 'OK: Found 1 TSS instance'
}

test_check_cip() {
  check_probe 'check_cip' '' 'ok' 'WARNING: CIP works, but does not process information'
  ./deployment/unicore-servers-*/unicorex/bin/stop.sh > /dev/null 2>&1
  check_probe 'check_cip' '' 'ok' 'CRITICAL: CIP does not respond.'
  sed -i -e's/uas.onstartup.2/#uas.onstartup.2/' ./deployment/unicore-servers-*/unicorex/conf/uas.config
  rm -rf ./deployment/unicore-servers-*/unicorex/data/*
  ./deployment/unicore-servers-*/unicorex/bin/start.sh > /dev/null 2>&1
  sleep 20
  check_probe 'check_cip' '' 'ok' 'WARNING: CIP responds but seems to be not initialized.'

  ./deployment/unicore-servers-*/unicorex/bin/stop.sh > /dev/null 2>&1
  sed -i -e's/#uas.onstartup.2/uas.onstartup.2/' ./deployment/unicore-servers-*/unicorex/conf/uas.config
  rm -rf ./deployment/unicore-servers-*/unicorex/data/*
  ./deployment/unicore-servers-*/unicorex/bin/start.sh > /dev/null 2>&1
  sleep 20
}

test_check_application() {
  DATE_FILE=`pwd`/localConfiguration/check_application/date.u
  check_probe 'check_application' "$DATE_FILE 'not_empty(#stdout)'" 'ok' 'OK: Date test job succeded, time elapsed: ' 
  ./deployment/unicore-servers-*/unicorex/bin/stop.sh > /dev/null 2>&1
  check_probe 'check_application' "$DATE_FILE 'not_empty(#stdout)'" 'ok' 'CRITICAL: TSS is not available!' 
  ./deployment/unicore-servers-*/unicorex/bin/start.sh > /dev/null 2>&1
  sleep 50
  check_probe 'check_application' "$DATE_FILE \"(not_empty_1(#stdout)\"" 'ok' 'UNKNOWN: Wrong condition syntax! See README file.' 
  check_probe 'check_application' "$DATE_FILE \"grep(NOT_THE_DATE,#stdout)\"" 'ok' 'WARNING: Unable to pass application condition string.' 
}

test_check_storagefactory() {
  ./deployment/unicore-servers-*/unicorex/bin/stop.sh > /dev/null 2>&1
  check_probe 'check_storagefactory' '' 'ok' 'CRITICAL: Unable to create storage:'
  rm -rf ./deployment/unicore-servers-*/unicorex/data/*
  ./deployment/unicore-servers-*/unicorex/bin/start.sh > /dev/null 2>&1
  sleep 30
  check_probe 'check_storagefactory' '' 'ok' 'OK: Storage factory at .* works'
}

test_check_sms() {
  ./deployment/unicore-servers-*/unicorex/bin/stop.sh > /dev/null 2>&1
  check_probe 'check_sms' '' 'ok' 'CRITICAL: Unable to get access to the SMS at'
  rm -rf ./deployment/unicore-servers-*/unicorex/data/*
  ./deployment/unicore-servers-*/unicorex/bin/start.sh > /dev/null 2>&1
  sleep 30
  chmod 000 ./deployment/unicore-servers-*/unicorex/storage
  check_probe 'check_sms' '' 'ok' 'CRITICAL: Unable to write file at server side'
  chmod 755 ./deployment/unicore-servers-*/unicorex/storage
  check_probe 'check_sms' '' 'ok' 'OK: SMS at .* works with .*'
}

test_check_freespace() {
  ./deployment/unicore-servers-*/unicorex/bin/stop.sh > /dev/null 2>&1
  check_probe 'check_freespace' '' 'ok' 'UNKNOWN: Cannot get access to the SMS'
  rm -rf ./deployment/unicore-servers-*/unicorex/data/*
  ./deployment/unicore-servers-*/unicorex/bin/start.sh > /dev/null 2>&1
  sleep 30
  check_probe 'check_freespace' '-w 2 -c 1' 'ok' 'OK: There is .* of free space'
  check_probe 'check_freespace' '-w 999999999999999 -c 1' 'ok' 'WARNING: There is .* of free space'
  check_probe 'check_freespace' '-w 999999999999999 -c 999999999999998' 'ok' 'CRITICAL: There is .* of free space'
}

test_check_workflowservice() {
  ./deployment/unicore-workflow-*/workflow/bin/stop.sh > /dev/null 2>&1
  check_probe 'check_workflowservice' '' 'ok' 'CRITICAL: Workflow service .* seems to be down'
  rm -rf ./deployment/unicore-workflow-*/workflow/data/*
  ./deployment/unicore-workflow-*/workflow/bin/start.sh > /dev/null 2>&1
  sleep 30
  check_probe 'check_workflowservice' '' 'ok' 'OK: Workflow Service works normally, found workflows:'
}

test_check_servorch() {
  check_probe 'check_servorch' '' 'ok' 'OK: Service Orchestator appears to work, sites registered in GRIS:'
  ./deployment/unicore-workflow-*/servorch/bin/stop.sh > /dev/null 2>&1
  check_probe 'check_servorch' '' 'ok' 'CRITICAL: Unable to contact Service Orchestrator'
  rm -rf ./deployment/unicore-workflow-*/servorch/data/*
  ./deployment/unicore-servers-*/unicorex/bin/stop.sh > /dev/null 2>&1
  ./deployment/unicore-workflow-*/servorch/bin/start.sh > /dev/null 2>&1
  sleep 30
  check_probe 'check_servorch' '' 'ok' 'CRITICAL: Unable to find any resource in GRIS'
  ./deployment/unicore-servers-*/unicorex/bin/start.sh > /dev/null 2>&1
  sleep 40
}

start_unicore() {
  echo "Initializing servers..."
  ./deployment/uvos-*/bin/startServer.sh > /dev/null 2>&1
  ./deployment/unicore-servers-*/start.sh > /dev/null 2>&1
  ./deployment/unicore-workflow-*/servorch/bin/start.sh > /dev/null 2>&1
  ./deployment/unicore-workflow-*/workflow/bin/start.sh > /dev/null 2>&1
  sleep 60 # Time to initialize
}

stop_unicore() {
  echo "Stopping servers..."
  ./deployment/unicore-workflow-*/servorch/bin/stop.sh > /dev/null 2>&1
  ./deployment/unicore-workflow-*/workflow/bin/stop.sh > /dev/null 2>&1
  ./deployment/unicore-servers-*/stop.sh > /dev/null 2>&1
  ./deployment/uvos-*/bin/stopServer.sh > /dev/null 2>&1
}

start_unicore
test_check_gateway
test_check_uvos
test_check_registry
test_check_unicorex
test_check_cip
test_check_application
test_check_storagefactory
test_check_sms
test_check_freespace
test_check_workflowservice
test_check_servorch
stop_unicore
